package me.zeroeightsix.kami.module.modules.render

import me.zeroeightsix.kami.module.Module
import me.zeroeightsix.kami.setting.Setting
import me.zeroeightsix.kami.setting.Settings

/**
 * @author dominikaaaa
 * Created by dominikaaaa on 20/12/19
 * Updated by dominikaaaa on 31/05/20
 */
@Module.Info(
        name = "Zoom",
        category = Module.Category.RENDER,
        description = "Configures FOV",
        showOnArray = Module.ShowOnArray.OFF
)
class Zoom : Module() {
    private var fov = 0f
    private var sensi = 0f

    private val fovChange = register(Settings.integerBuilder("FOV").withRange(1, 130).withValue(40).build())
    private val sens = register(Settings.b("Change Sensitivity", true))
    private val sensChange = register(Settings.floatBuilder("Sensitivity Multiplier").withMinimum(0.25f).withValue(1.3f).withMaximum(2f).withVisibility { sens.value }.build())
    private val smoothCamera = register(Settings.b("Cinematic Camera", false))

    init {
        fovChange.settingListener = Setting.SettingListeners { if (isEnabled && mc.player != null) mc.gameSettings.fovSetting = fovChange.value.toFloat() }
        sens.settingListener = Setting.SettingListeners { if (isEnabled && mc.player != null) if (sens.value) mc.gameSettings.mouseSensitivity = sensi * sensChange.value else mc.gameSettings.mouseSensitivity = sensi }
        sensChange.settingListener = Setting.SettingListeners { if (isEnabled && mc.player != null) mc.gameSettings.mouseSensitivity = sensi * sensChange.value }
        smoothCamera.settingListener = Setting.SettingListeners { if (isEnabled && mc.player != null) mc.gameSettings.smoothCamera = smoothCamera.value }
    }

    public override fun onEnable() {
        if (mc.player == null) return
        fov = mc.gameSettings.fovSetting
        sensi = mc.gameSettings.mouseSensitivity

        mc.gameSettings.fovSetting = fovChange.value.toFloat()
        if (sens.value) mc.gameSettings.mouseSensitivity = sensi * sensChange.value
        mc.gameSettings.smoothCamera = smoothCamera.value
    }

    public override fun onDisable() {
        mc.gameSettings.fovSetting = fov
        mc.gameSettings.mouseSensitivity = sensi
        mc.gameSettings.smoothCamera = false
    }
}